#include "spline.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

struct point *arr;
int len;

void load_data();
void plotvals();
static void setup_ogl(int width, int height);

double xmin;
double xmax;
const double ymin = -0.1;
const double ymax =  1.1;

void setup_ogl (int width, int height)
{
   glShadeModel (GL_SMOOTH);
   glCullFace (GL_BACK);
   glFrontFace (GL_CCW);
   glEnable(GL_CULL_FACE);

   glClearColor (0,0,0,0);
  
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity();
	printf("x: %lf <-> %lf\n", xmin, xmax);
	printf("y: %lf <-> %lf\n", ymin, ymax);
	gluOrtho2D(xmin,xmax, ymin,ymax);
}

void load_data(int argc, char *argv[])
{
	int i;
	if(argc < 2) {
		printf("required argument: data file\n");
		exit(1);
	}
	arr = point_load(argv[1], &len);
	for(i=0; i < len; ++i) {
		printf("%8g at %3g\n", arr[i].p, arr[i].t);
	}
}

void plotvals()
{
	double t;
	struct point *p, q;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_LINES); /* border */
		glVertex2f(xmin+.001, ymin+0.001);
		glVertex2f(xmin+.001, ymax-0.001);

		glVertex2f(xmax-.001, ymin+0.001);
		glVertex2f(xmax-.001, ymax-0.001);

		glVertex2f(xmin+.001, ymin+0.001);
		glVertex2f(xmax-.001, ymin+0.001);

		glVertex2f(xmin+.001, ymax-0.001);
		glVertex2f(xmax-.001, ymax-0.001);
	glEnd();

	for(t = 0.0; t < 1.0; t += 0.001) {
		p = point_find(arr, len, t);

		glBegin(GL_LINES);
			glColor3f(0.0, 0.0, 1.0);
			glVertex2d(p[0].p, p[0].t);

			glColor3f(0.0, 1.0, 0.0);
			glVertex2d(p[1].p, p[1].t);
		glEnd();
		glColor3f(1.0, 0.0, 1.0);

		q = point_generate(p, t);
		glBegin(GL_POINTS);
			glVertex2d(q.p, t);
		glEnd();

		glFlush();
		glutSwapBuffers();
		usleep(900);
	}
}

void idle()
{
	usleep(999);
}

int main(int argc, char *argv[])
{
	int imageWindowID;
	double c_min, c_max;
	int i;
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowPosition (800, 240);
	glutInitWindowSize(320,240);
	imageWindowID = glutCreateWindow(argv[0]);

	load_data(argc, argv);

	c_min = c_max = arr[0].p;
	for(i=0; i < len; ++i) {
		if(arr[i].p < c_min) {
			c_min = arr[i].p;
		}
		if(arr[i].p > c_max) {
			c_max = arr[i].p;
		}
	} 
	xmin = c_min;
	xmax = c_max;
	setup_ogl(320, 240);

	glutIdleFunc(idle);

	glutDisplayFunc(plotvals);
	glutMainLoop();
	return 0;
}
