/*
 * Copyright (c) 2005, Ethan A. Burns <eaburns@unh.edu>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _EMF_H_
#define _EMF_H_

#include <stdint.h>

/* This is just for the future when we may not be able to count on 
 * the size of a float. */
typedef float float32_t;

struct emf_vertex {
  float32_t x, y, z;
  float32_t nx, ny, nz;
} __attribute__((packed));

struct emf_color {
  float32_t r, g, b, a;
} __attribute__((packed));

struct emf_face {
  uint32_t n;           /* Number of vertices in this face. */
  uint32_t indices[4];  /* The vertex indices. */

  /* Material properties. */
  struct emf_color specular;
  struct emf_color color;
  uint32_t shininess;

} __attribute__((packed));

struct emf_frame {
  uint32_t nverts, nfaces;
  struct emf_vertex *verts;
  struct emf_face *faces;
} __attribute__((packed));

struct emf {
  uint32_t flags;
  uint32_t nframes;
  struct emf_frame *frames;
  struct emf_frame *curframe;
} __attribute__((packed));

/* Loads the emf model into the buffer from the file given by path. */
int load_emf( struct emf *buff, const char *path );

/* Frees all the memory associated with the emf model. */
void unload_emf( struct emf *buff );

/* Draws the emf model using openGL calls. */
void draw_emf( struct emf *buff );

#endif /* _EMF_H_ */
