/*
 * Copyright (c) 2005, Ethan A. Burns <eaburns@unh.edu>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <unistd.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include <emf.h>

#define WIDTH 800
#define HEIGHT 600
#define COLORS 16

static void setup_ogl( int width, int height )
{
  GLfloat light_diffuse[] = { 0.9, 0.9, 0.9, 1.0 };
  GLfloat light_specular[] = { 0.5, 0.5, 0.5, 1.0 };
  GLfloat light_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
  GLfloat light_position[] = { 20.0 , 20.0, -10.0, 0.0 };
  glClearColor( 0, 0, 0, 0 );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();             
  gluPerspective( 60.0, (GLfloat)width/(GLfloat)height, 1.0, 150.0 );
  glViewport( 0, 0, width, height );
  glShadeModel( GL_SMOOTH );

#if 0
  /* Un-comment this to enable wire-frame */
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
#endif

  glEnable( GL_DEPTH_TEST );
  glEnable( GL_LIGHTING );
  glEnable( GL_LIGHT0 );
  glEnable( GL_LIGHT1 );
  glLightfv( GL_LIGHT0, GL_DIFFUSE, light_diffuse );
  glLightfv( GL_LIGHT0, GL_SPECULAR, light_specular );
  glLightfv( GL_LIGHT0, GL_AMBIENT, light_ambient );
  glLightfv( GL_LIGHT0, GL_POSITION, light_position );

  SDL_GL_SetAttribute (SDL_GL_DEPTH_SIZE, 16);
}

int main( int argc, char **argv )
{
  SDL_Surface *screen;
  SDL_Event event;
  int angle = 0;
  long time;
  int frames;
  struct emf model;
  char *path;

  if( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf( stderr, "Unable to initialize SDL: %s\n", SDL_GetError() );
    exit( -1 );
  }
  atexit( SDL_Quit );

  /* Use double buffering with opengl. */
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

  /* Grab a window with the desired dimentions using opengl and hardware
   * display surfaces. */
  if( (screen = SDL_SetVideoMode( WIDTH, HEIGHT, COLORS,
       SDL_OPENGL|SDL_HWSURFACE )) == NULL ) {
    fprintf( stderr, "Unable to set video mode %dx%dx%d: %s\n", WIDTH, HEIGHT,
             COLORS, SDL_GetError() );
    exit( -1 );
  }
  setup_ogl( WIDTH, HEIGHT );

  if( argc == 2 ) {
    path = argv[1];
  } else {
    fprintf( stderr, "Please specify .emf file to load\n" );
    return -1;
  }
    

  if( load_emf( &model, path ) != 0 ) { return -1; }

  glMatrixMode( GL_MODELVIEW );
  frames = 0; 
  time = SDL_GetTicks();
  do {
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    glLoadIdentity();
    glTranslatef( 0.0, 0.0, -40.0 );
    glRotatef( angle%360, 1.0, 1.0, 0.0 );

    draw_emf( &model );

    SDL_GL_SwapBuffers();
    angle = (angle+1)%360;
    usleep( 5000 );
    SDL_PumpEvents();

    /* Count fps. */
    frames++;
    if( SDL_GetTicks() >= time+1000 ) {
      fprintf( stderr, "\rfps: %d", frames );
      frames = 0;
      time = SDL_GetTicks();
    }

  } while( !(SDL_PollEvent( &event )) || event.type != SDL_KEYDOWN );
  fprintf( stderr, "\n" );

  unload_emf( &model );

  return 0;
}
