/*
Copyright (C) 1999-2006 Id Software, Inc. and contributors.
For a list of contributors, see the accompanying AUTHORS file.

This file is part of MCC.

MCC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MCC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MCC; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//-----------------------------------------------------------------------------
//
//
// DESCRIPTION:
// deal with in/out tasks, for either stdin/stdout or network/XML stream
// 

#include "cmdlib.h"
#include "mathlib.h"
#include "polylib.h"
#include "inout.h"
#include <sys/types.h>
#include <sys/stat.h>

#ifdef WIN32
#include <direct.h>
#include <windows.h>
#endif

#ifdef WIN32
HWND hwndOut = NULL;
qboolean lookedForServer = false;
UINT wm_BroadcastCommand = -1;
#endif

qboolean verbose = false;

#define WINDING_BUFSIZE 2048

#include "stream_version.h"

void Broadcast_Setup( const char *dest )
{
}

void Broadcast_Shutdown()
{
}

// all output ends up through here
void FPrintf(int flag, char *buf)
{
  char level[2];

  printf(buf);

  // the following part is XML stuff only.. but maybe we don't want that message to go down the XML pipe?
  if (flag == SYS_NOXML)
    return;

  level[0] = (int)'0' + flag;
  level[1] = 0;
}

#ifdef DBG_XML
/* made empty; had a libxml call in it */
void DumpXML() { }
#endif

void Sys_FPrintf(int flag, const char *format, ...)
{
  char out_buffer[4096];
	va_list argptr;
  
  if ((flag == SYS_VRB) && (verbose == false))
    return;

  va_start (argptr, format);
	vsprintf (out_buffer, format, argptr);
	va_end (argptr);

  FPrintf (flag, out_buffer);
}

void Sys_Printf (const char *format, ...)
{
  char out_buffer[4096];
	va_list argptr;

  va_start (argptr, format);
	vsprintf (out_buffer, format, argptr);
	va_end (argptr);
  
  FPrintf (SYS_STD, out_buffer);
}

/*
=================
Error

For abnormal program terminations
=================
*/
void Error( const char *error, ...)
{
  char out_buffer[4096];
  char tmp[4096];
	va_list argptr;

	va_start (argptr,error);
	vsprintf (tmp, error, argptr);
	va_end (argptr);

  sprintf( out_buffer, "************ ERROR ************\n%s\n", tmp );

  FPrintf( SYS_ERR, out_buffer );

#ifdef DBG_XML  
  DumpXML();
#endif

  //++timo HACK ALERT .. if we shut down too fast the xml stream won't reach the listener.
  // a clean solution is to send a sync request node in the stream and wait for an answer before exiting
  Sys_Sleep( 1000 );
  
  Broadcast_Shutdown();

	exit (1);
}

