#include <kthread.h>
#include <paging.h>
#include <sched.h>    /* sched_start_proc */
#include <ktypes.h>
#include <kalloc.h>
#include <tss.h>
#include <i386paging.h>
#include <i386proc.h>
#include <memman.h>
#include <retval.h>
#include <descriptor.h>
#include <kstdio.h>
#include <memutil.h>
#include <memspace.h>

#define DATA_LEN 0x8000 /* kthread stack size. */
int kthread_create( kthread_t *p, void* text )
{
  uint32* esp;
  addr_space_t as;

  p->kcode = 1; /* Execute at kernel level. */

  if( alloc_kern_space( &as, DATA_LEN ) == RET_FAILURE ) {
    kprintf( "Unable to allocate kthread address space." );
    return RET_FAILURE;
  }

	esp = (uint32*)virt_to_physical( as.pd, MEMSPACE_USER+DATA_LEN ); 
	esp--;
	*esp = (uint32)curthread_kill; 

  /* Copy over the address space to the process struct. */
  p->addr_space = kalloc( sizeof(addr_space_t) );
  memcpy( p->addr_space, &as, sizeof(addr_space_t) );

  /* Allocate a tss_t for the kthread. */
  p->arch_info = kalloc( sizeof(proc_info_t) );
  ((proc_info_t*)p->arch_info)->tss = kalloc( sizeof(tss_t) );
  bzero( ((proc_info_t*)p->arch_info)->tss, sizeof(tss_t) );

  /* It's okay to use the physical address here... but this should be fixed
   * later to be the virtual address. */
  ((proc_info_t*)p->arch_info)->tss->esp = MEMSPACE_USER+DATA_LEN-4;
  ((proc_info_t*)p->arch_info)->tss->ss = KDATA_SELECTOR;

/*
  ((proc_info_t*)p->arch_info)->tss->esp0 = MEMSPACE_USER+(DATA_LEN/2);
  ((proc_info_t*)p->arch_info)->tss->ss0 = KDATA_SELECTOR;
*/

  ((proc_info_t*)p->arch_info)->tss->es = KDATA_SELECTOR;
  ((proc_info_t*)p->arch_info)->tss->ds = KDATA_SELECTOR;
  ((proc_info_t*)p->arch_info)->tss->fs = KDATA_SELECTOR;
  ((proc_info_t*)p->arch_info)->tss->gs = KDATA_SELECTOR;
  ((proc_info_t*)p->arch_info)->tss->cs = KCODE_SELECTOR;

  ((proc_info_t*)p->arch_info)->tss->eip = (uint32)text;

  ((proc_info_t*)p->arch_info)->tss->eflags = 0x200;
  ((proc_info_t*)p->arch_info)->tss->cr3 = (word_t)p->addr_space->pd;

  if( sched_add( (process_t*)p ) == RET_FAILURE ) {
    kprintf( "Failed to add kernel thread to scheduler.\n" );
    free_addr_space( p->addr_space );
    kthread_destroy( p ); 
    return RET_FAILURE;
  }

  return RET_SUCCESS;
}

/* This destroy function must free all kalloced memory from kthread_create. */
void kthread_destroy( kthread_t *p )
{
  kfree( ((proc_info_t*)p->arch_info)->tss );
  kfree( p->arch_info );
  kfree( p->addr_space );
}
