#include <paging.h>
#include <process.h>
#include <sched.h>    /* sched_start_proc */
#include <ktypes.h>
#include <kalloc.h>
#include <tss.h>
#include <i386paging.h>
#include <i386proc.h>
#include <memman.h>
#include <retval.h>
#include <descriptor.h>
#include <kstdio.h>
#include <memutil.h>
#include <memspace.h>

#define DATA_LEN 0x10000
int create_process( process_t *p, void* text, uint32 t_len )
{
  uint32 i, addr;
  uint8 *t_pointer;
  addr_space_t as;

  p->kcode = 0; /* Execute at user level. */

  if( alloc_addr_space( &as, sched_start_proc_len+t_len+DATA_LEN) == 
                                                              RET_FAILURE ) {
    kprintf( "Unable to allocate process address space." );
    return RET_FAILURE;
  }

  /* Copy over the address space to the process struct. */
  p->addr_space = kalloc( sizeof(addr_space_t) );
  memcpy( p->addr_space, &as, sizeof(addr_space_t) );
  
  t_pointer = text;
  addr = MEMSPACE_USER;
  for( i = 0; i < t_len; i++ ) {
    *((uint8*)virt_to_physical( as.pd, (word_t)addr )) = *t_pointer;
    addr++, t_pointer++;
  }

  /* Copy our start process, asm, wrapper routine to user memory. */
  memcpy( (void*)virt_to_physical( as.pd, MEMSPACE_USER+t_len+DATA_LEN ),
          sched_start_proc, sched_start_proc_len );

  /* Allocate a tss_t for the process. */
  p->arch_info = kalloc( sizeof(proc_info_t) );
  ((proc_info_t*)p->arch_info)->tss = kalloc( sizeof(tss_t) );
  bzero( ((proc_info_t*)p->arch_info)->tss, sizeof(tss_t) );

  ((proc_info_t*)p->arch_info)->tss->esp0 = MEMSPACE_USER+t_len+DATA_LEN;
  ((proc_info_t*)p->arch_info)->tss->ss0 = KDATA_SELECTOR;

  ((proc_info_t*)p->arch_info)->tss->esp = t_len+(DATA_LEN/2);
  ((proc_info_t*)p->arch_info)->tss->ss = UDATA_SELECTOR|3;

  ((proc_info_t*)p->arch_info)->tss->es = UDATA_SELECTOR|3;
  ((proc_info_t*)p->arch_info)->tss->ds = UDATA_SELECTOR|3;
  ((proc_info_t*)p->arch_info)->tss->fs = UDATA_SELECTOR|3;
  ((proc_info_t*)p->arch_info)->tss->gs = UDATA_SELECTOR|3;

  ((proc_info_t*)p->arch_info)->tss->cs = UCODE_SELECTOR|3;

  ((proc_info_t*)p->arch_info)->tss->eip = t_len+DATA_LEN;
  ((proc_info_t*)p->arch_info)->tss->eflags = 0x200;
  ((proc_info_t*)p->arch_info)->tss->cr3 = (word_t)p->addr_space->pd;

  if( sched_add( p ) == RET_FAILURE ) {
    kprintf( "Failed to add process to scheduler.\n" );
    free_addr_space( p->addr_space );
    destroy_process( p ); 
    return RET_FAILURE;
  }

  return RET_SUCCESS;
}

/* This destroy function must free all kalloced memory from create_process. */
void destroy_process( process_t *p )
{
  kfree( ((proc_info_t*)p->arch_info)->tss );
  kfree( p->arch_info );
  kfree( p->addr_space );
}
