/** Routines concerning kernel semaphore operations.
 * Currently ix86-only -- requires the 'xchg' instruction! */
#include <sem.h>
#include <queue.h>
#include <sched.h>
#include <kstdlib.h>

static void mutex_init(mutex_t *s);
extern void mutex_lock(mutex_t *);
static void mutex_unlock(mutex_t *);

/** initializes a mutex. doesn't acquire the appropriate lock -- use with
 * caution! */
static void mutex_init(mutex_t *m) {
	m->lock = 1;
	queue_init(m->wait_q);
}

/** unlocks a binary semaphore. */
static void mutex_unlock(mutex_t *m) {
	/* might need to use asm to make this atomic.
	 * but 'inc' is atomic and unless the compiler is dumb enough to use 'add'
	 * or something, we should be okay... */
#if 0
	m->value++;
#else
  /* The [] notation does not work on gcc 2.9.5 of course :( */
/*	__asm__("incl %[mut]\n\t" : : [mut] "m" (m->lock) : "memory","cc"); */
	__asm__("incl %0\n\t" : : "m" (m->lock) : "memory","cc"); 

#endif

  /* This should make the unlock work for even non-mutex semaphores. */
  if( m->lock <= 0 ) { return; }

  /* Wakeup any waiting processes. */
  if(m->wait_q) {
    wake( m->wait_q ); 
  }

}

/** initializes a semaphore -- must be called before trying to use the
 * semaphore */
void sem_init(sem_t *s, uint32 r) {
	mutex_init(&s->m);
	s->value = r;
  queue_init( s->wait_q );
}

/** locks a semaphore */
void sem_lock(sem_t *s) {

	while(s->value <= 0)
    wait( s->wait_q );

	mutex_lock(&s->m);
		s->value--;
	mutex_unlock(&s->m);

}

void sem_unlock(sem_t *s) {
	mutex_lock(&s->m);
		s->value++;
	mutex_unlock(&s->m);

	if(s->value > 0)
    wake( s->wait_q );
}
