#ifndef _FS_H_
#define _FS_H_

#include <ktypes.h>
#include <common/types.h>
#include <device.h>

/* file_t is a structure used to represent an open file in a process's open
 * file table. */
typedef struct {
  char *name;       /* Just name, not path? or full path? */
  off_t file_ptr;  /* read/write offset. */

  uint32 refs;      /* Files can share open files?  (fork anyone?) */

} file_t;

/* This should be a general inode.  This means that all file systems will
 * use this as the in-memory inode and we need the union of file system 
 * specific inode information. */
typedef uint32 efs_inode_info;    /* Eventually will be a struct in efs code. */

typedef struct {
  size_t blk_size;  /* specific to the fs used. */ 

  size_t size;      /* In bytes. */
  uint32 refs;
  uid_t uid;
  gid_t gid;
  mode_t mode;      /* O_RDONLY/O_RDWR/etc. */
  uint32 perm;      /* user/group/all read/write/execute privledges, dir flag
                     * suid flag, etc. */
  
  time_t atime;     /* Time of last access. */
  time_t ctime;     /* Time created. */
  time_t mtime;     /* Time of last modification. */

  /* Maybe throw in a semaphore. */


  union {
    efs_inode_info efs_inode;
  } fs_spec;

} inode_t;

typedef struct {
} super_blk_t;

typedef struct {
  char *type;             
  /* Needs a function to read the superblock.  From there, the super block
   * can have some read/write operations for inodes... inodes can have 
   * read/write operations for files... all the way down the heirarchy. */
} fs_t;                     /* Name of the file system. */

/* File operations struct holds file operations common to all devices. */
typedef struct {
  int (*lseek)(inode_t *inode, file_t *file, off_t offset, int whence);
  int (*read)(inode_t *inode, file_t *file, void* buff, size_t count);
  int (*write)(inode_t *inode, file_t *file, void* buff, size_t count);
/* int (*readdir)(inode_t *inode, file_t *file, struct dirent *dirp, 
                  unsigned int count); */
/* int (*select)(?); */
/* int (*ioctl)(file_t *file, int request, ...?) */
  int (*mmap)(inode_t *inode, file_t *file, void* start, size_t length, 
              int prot, int flags, off_t offset);
              
  int (*open)(inode_t *inode, file_t *file);
} file_ops_t;

#endif /* _FS_H_ */
