#ifndef _SCHED_H_
#define _SCHED_H_

#ifndef __ASM__
#	include <process.h>
#	include <ktypes.h>
# include <queue.h>
#endif /* !__ASM__ */

/* Maximum number of processes... this should be moved to .config some time. */
#ifndef SCHED_MAX_PROC 
#define SCHED_MAX_PROC 5
#endif /* SCHED_MAX_PROCS */

#ifndef __ASM__
/* Initializes the scheduler. */
int init_sched();

/* Start and stop the scheduler. */
void scheduler_start();
void scheduler_stop();

/* Add a process to the scheduler. */
int sched_add( process_t *p );

/* Context switches to the next process on the list. */
void schedule();

/* Kills a process with the given pid.  This function will free-up the pages
 * associated with the given process that it is freeing. */
void process_kill( uint32 pid );

/* Sched start proc is a small assembly routine to call a process.
 * This is here so when a process returns we have a handle to kill that
 * process. */
void sched_start_proc();
/* Length of the wrapper function in bytes. */
extern uint32 sched_start_proc_len;

/* Performs the actual context switch, this is arch dependent. */
void context_switch( process_t *to, process_t *from );

/* Arch specific scheduler add process. */
void arch_sched_add( process_t *p );

/* Adds the current process to the tail of the wait queue, and sets
 * its state to wait, finally the scheduler is called and the next
 * process is started. */
void wait( struct queue *wq );

/* Wakes the process at the head of the queue.  Removes the process 
 * from the queue, sets its state to running and returns.  This 
 * function does not call the scheduler. 
 * The return value is a pointer to the process that was just woken
 * up. */
process_t *wake( struct queue *wq );

/* Currently running pid. */
extern uint32 curproc;

#else
	.extern curproc
#endif /* !__ASM__ */

#endif /* !_SCHED_H_ */
