/** Simple queue for various purposes throughout the operating system.
 * Note the queues defined here are NOT THREAD SAFE (mostly because they were
 * first written to implement semaphore queues...) */
#ifndef QUEUE_H
#define QUEUE_H

#include <ktypes.h>

typedef void* qnode; /* defined in implementation */

struct queue {
	struct qnode *head;
	struct qnode *tail;
} queue_t;

/** initializes a queue element by setting all its pointers to NULL. */
void queue_init(struct queue *);

/** adds a new element with the specified data to the end */
void queue_add(struct queue *, void *);

/** pops off the head element of the queue, and resets the queue's head to be
 * the elem right after that */
void *queue_pop(struct queue *);

/* Returns the tail element of the queue, meant for compairsons only. */
void* queue_tail( struct queue *q );

/* Returns 1 if the queue is empty, or else it returns a 0. */
uint32 queue_empty( struct queue *q );

/* Returns the head element of the queue, with out removing it from the 
 * queue. */
void* queue_head( struct queue *q );

#endif /* QUEUE_H */
