/** Simple queue for various purposes throughout the operating system.
 * Note the queues defined here are NOT THREAD SAFE (mostly because they were
 * first written to implement semaphore queues...) */
#include "queue.h"
#include "kstdlib.h"
#include "kalloc.h"
#include "panic.h"
#include <kstdio.h>
#include <kstdlib.h>

struct qnode {
	void *data;
	struct qnode *next;
} qnode_t;

/** initializes a queue element by setting all its pointers to NULL. */
void queue_init(struct queue *q) {
	q->head = NULL;
	q->tail = NULL;
}

void queue_add(struct queue *q, void *data) {
	struct qnode *elem;

	elem = kalloc(sizeof(struct qnode));
	if(!elem) {
		panic("Allocation for kernel queue memory failed!\n");
	}

  elem->data = data;

	if(!q->head) { /* first insert */
		q->head = q->tail = elem;
	} else { /* add to tail */
		q->tail->next = elem;
		q->tail = q->tail->next;
	}

	q->tail->next = NULL;
}

void *queue_pop(struct queue *q) {
	void *data;
	struct qnode *prev_head;

	if(q->head) {

		data = q->head->data;
		prev_head = q->head;
		if(q->head == q->tail) { /* removing the only elem */
			q->head = q->tail = NULL;
		} else {
			q->head = q->head->next;
		}

		kfree(prev_head);
	} else {

    return NULL;
	}

	return data;
}

void* queue_tail( struct queue *q )
{
  return q->tail;
}

uint32 queue_empty( struct queue *q )
{
  return q->head == NULL? 1 : 0;
}

void* queue_head( struct queue *q )
{
  return q->head;
}
